@@ -1,5 +1,5 @@
 /* Support for cookies.
-   Copyright (C) 2001-2011, 2015, 2018-2019 Free Software Foundation,
+   Copyright (C) 2001-2011, 2015, 2018-2020 Free Software Foundation,
    Inc.
 
 This file is part of GNU Wget.
@@ -395,12 +395,15 @@ parse_set_cookie (const char *set_cookie, bool silent)
         }
       else if (TOKEN_IS (name, "expires"))
         {
-          char *value_copy;
+          char value_copy[128];
+          size_t value_len = value.e - value.b;
           time_t expires;
 
-          if (!TOKEN_NON_EMPTY (value))
+          if (!TOKEN_NON_EMPTY (value) || value_len >= sizeof (value_copy))
             goto error;
-          BOUNDED_TO_ALLOCA (value.b, value.e, value_copy);
+
+          memcpy (value_copy, value.b, value_len);
+          value_copy[value_len] = 0;
 
           /* Check if expiration spec is valid.
              If not, assume default (cookie doesn't expire, but valid only for
@@ -420,18 +423,21 @@ parse_set_cookie (const char *set_cookie, bool silent)
       else if (TOKEN_IS (name, "max-age"))
         {
           double maxage = -1;
-          char *value_copy;
+          char value_copy[32];
+          size_t value_len = value.e - value.b;
 
-          if (!TOKEN_NON_EMPTY (value))
+          if (!TOKEN_NON_EMPTY (value) || value_len >= sizeof (value_copy))
             goto error;
-          BOUNDED_TO_ALLOCA (value.b, value.e, value_copy);
+
+          memcpy (value_copy, value.b, value_len);
+          value_copy[value_len] = 0;
 
           sscanf (value_copy, "%lf", &maxage);
           if (maxage == -1)
             /* something went wrong. */
             goto error;
           cookie->permanent = 1;
-          cookie->expiry_time = cookies_now + maxage;
+          cookie->expiry_time = cookies_now + (time_t) maxage;
 
           /* According to rfc2109, a cookie with max-age of 0 means that
              discarding of a matching cookie is requested.  */
@@ -521,14 +527,15 @@ numeric_address_p (const char *addr)
    upper case strings.
    */
 
+#ifdef HAVE_LIBPSL
+static psl_ctx_t *psl;
+#endif
+
 static bool
 check_domain_match (const char *cookie_domain, const char *host)
 {
-
 #ifdef HAVE_LIBPSL
   static int init_psl;
-  static const psl_ctx_t *psl;
-
   char *cookie_domain_lower = NULL;
   char *host_lower = NULL;
   int is_acceptable;
@@ -722,17 +729,6 @@ check_path_match (const char *cookie_path, const char *path)
   return path_matches (path, cookie_path) != 0;
 }
 
-/* Prepend '/' to string S.  S is copied to fresh stack-allocated
-   space and its value is modified to point to the new location.  */
-
-#define PREPEND_SLASH(s) do {                                   \
-  char *PS_newstr = (char *) alloca (1 + strlen (s) + 1);       \
-  *PS_newstr = '/';                                             \
-  strcpy (PS_newstr + 1, s);                                    \
-  s = PS_newstr;                                                \
-} while (0)
-
-
 /* Process the HTTP `Set-Cookie' header.  This results in storing the
    cookie or discarding a matching one, or ignoring it completely, all
    depending on the contents.  */
@@ -744,11 +740,20 @@ cookie_handle_set_cookie (struct cookie_jar *jar,
 {
   struct cookie *cookie;
   cookies_now = time (NULL);
+  char buf[1024], *tmp;
+  size_t pathlen = strlen(path);
 
   /* Wget's paths don't begin with '/' (blame rfc1808), but cookie
      usage assumes /-prefixed paths.  Until the rest of Wget is fixed,
      simply prepend slash to PATH.  */
-  PREPEND_SLASH (path);
+  if (pathlen < sizeof (buf) - 1)
+    tmp = buf;
+  else
+    tmp = xmalloc (pathlen + 2);
+
+  *tmp = '/';
+  memcpy (tmp + 1, path, pathlen + 1);
+  path = tmp;
 
   cookie = parse_set_cookie (set_cookie, false);
   if (!cookie)
@@ -810,11 +815,15 @@ cookie_handle_set_cookie (struct cookie_jar *jar,
     }
 
   store_cookie (jar, cookie);
+  if (tmp != buf)
+    xfree (tmp);
   return;
 
  out:
   if (cookie)
     delete_cookie (cookie);
+  if (tmp != buf)
+    xfree (tmp);
 }
 
 /* Support for sending out cookies in HTTP requests, based on
@@ -1046,28 +1055,47 @@ char *
 cookie_header (struct cookie_jar *jar, const char *host,
                int port, const char *path, bool secflag)
 {
-  struct cookie **chains;
+  struct cookie *chains[32];
   int chain_count;
 
   struct cookie *cookie;
   struct weighed_cookie *outgoing;
   size_t count, i, ocnt;
-  char *result;
+  char *result = NULL;
   int result_size, pos;
-  PREPEND_SLASH (path);         /* see cookie_handle_set_cookie */
+  char pathbuf[1024];
 
   /* First, find the cookie chains whose domains match HOST. */
 
   /* Allocate room for find_chains_of_host to write to.  The number of
      chains can at most equal the number of subdomains, hence
-     1+<number of dots>.  */
-  chains = alloca_array (struct cookie *, 1 + count_char (host, '.'));
+     1+<number of dots>.  We ignore cookies with more than 32 labels. */
+  chain_count = 1 + count_char (host, '.');
+  if (chain_count > (int) countof (chains))
+    return NULL;
   chain_count = find_chains_of_host (jar, host, chains);
 
   /* No cookies for this host. */
   if (chain_count <= 0)
     return NULL;
 
+  /* Wget's paths don't begin with '/' (blame rfc1808), but cookie
+     usage assumes /-prefixed paths.  Until the rest of Wget is fixed,
+     simply prepend slash to PATH.  */
+  {
+    char *tmp;
+    size_t pathlen = strlen(path);
+
+    if (pathlen < sizeof (pathbuf) - 1)
+      tmp = pathbuf;
+    else
+      tmp = xmalloc (pathlen + 2);
+
+    *tmp = '/';
+    memcpy (tmp + 1, path, pathlen + 1);
+    path = tmp;
+  }
+
   cookies_now = time (NULL);
 
   /* Now extract from the chains those cookies that match our host
@@ -1081,11 +1109,11 @@ cookie_header (struct cookie_jar *jar, const char *host,
       if (cookie_matches_url (cookie, host, port, path, secflag, NULL))
         ++count;
   if (!count)
-    return NULL;                /* no cookies matched */
+    goto out;                /* no cookies matched */
 
   /* Allocate the array. */
   if (count > SIZE_MAX / sizeof (struct weighed_cookie))
-    return NULL;                /* unable to process so many cookies */
+    goto out;                /* unable to process so many cookies */
   outgoing = xmalloc (count * sizeof (struct weighed_cookie));
 
   /* Fill the array with all the matching cookies from the chains that
@@ -1148,7 +1176,12 @@ cookie_header (struct cookie_jar *jar, const char *host,
   result[pos++] = '\0';
   xfree (outgoing);
   assert (pos == result_size);
-  return result;
+
+out:
+  if (path != pathbuf)
+    xfree (path);
+
+return result;
 }
 
 /* Support for loading and saving cookies.  The format used for
@@ -1312,7 +1345,7 @@ cookie_jar_load (struct cookie_jar *jar, const char *file)
         {
           if (expiry < cookies_now)
             goto abort_cookie;  /* ignore stale cookie. */
-          cookie->expiry_time = expiry;
+          cookie->expiry_time = (time_t) expiry;
           cookie->permanent = 1;
         }
 
@@ -1349,7 +1382,7 @@ cookie_jar_save (struct cookie_jar *jar, const char *file)
       return;
     }
 
-  fputs ("# HTTP cookie file.\n", fp);
+  fputs ("# HTTP Cookie File\n", fp);
   fprintf (fp, "# Generated by Wget on %s.\n", datetime_str (cookies_now));
   fputs ("# Edit at your own risk.\n\n", fp);
 
@@ -1411,6 +1444,11 @@ cookie_jar_delete (struct cookie_jar *jar)
     }
   hash_table_destroy (jar->chains);
   xfree (jar);
+
+#ifdef HAVE_LIBPSL
+  psl_free (psl);
+  psl = NULL;
+#endif
 }
 
 /* Test cases.  Currently this is only tests parse_set_cookies.  To
