@@ -1,6 +1,6 @@
 /* Reading/parsing the initialization file.
-   Copyright (C) 1996-2012, 2014-2015, 2018 Free Software Foundation,
-   Inc.
+   Copyright (C) 1996-2012, 2014-2015, 2018-2020 Free Software
+   Foundation, Inc.
 
 This file is part of GNU Wget.
 
@@ -58,6 +58,8 @@ as that of the covered work.  */
 #include "host.h"
 #include "netrc.h"
 #include "progress.h"
+#include "connect.h"            /* for connect_cleanup */
+#include "ssl.h"                /* for ssl_cleanup */
 #include "recur.h"              /* for INFINITE_RECURSION */
 #include "convert.h"            /* for convert_cleanup */
 #include "res.h"                /* for res_cleanup */
@@ -330,7 +332,9 @@ static const struct {
   { "tries",            &opt.ntry,              cmd_number_inf },
   { "trustservernames", &opt.trustservernames,  cmd_boolean },
   { "unlink",           &opt.unlink_requested,  cmd_boolean },
+#ifndef __VMS
   { "useaskpass" ,      &opt.use_askpass,       cmd_use_askpass },
+#endif
   { "useproxy",         &opt.use_proxy,         cmd_boolean },
   { "user",             &opt.user,              cmd_string },
   { "useragent",        NULL,                   cmd_spec_useragent },
@@ -585,20 +589,33 @@ wgetrc_env_file_name (void)
   return NULL;
 }
 
+/* Append file name to (locally appropriate) directory spec.
+   Return pointer to allocated storage. */
+char *
+ajoin_dir_file (const char *dir, const char *file)
+{
+  char *dir_file;
+#ifdef __VMS
+   /* No separator: "dev:[dir]" + "name.type" */
+  dir_file = aprintf ("%s%s", dir, file);
+#else /* def __VMS */
+  /* Slash separator: "/a/b" + "/" + "name.type" */
+  dir_file = aprintf ("%s/%s", dir, file);
+#endif /* def __VMS [else] */
+  return dir_file;
+}
+
 /* Check for the existence of '$HOME/.wgetrc' and return its path
    if it exists and is set.  */
 char *
 wgetrc_user_file_name (void)
 {
   char *file = NULL;
-  /* If that failed, try $HOME/.wgetrc (or equivalent).  */
 
-#ifdef __VMS
-  file = "SYS$LOGIN:.wgetrc";
-#else /* def __VMS */
-  if (opt.homedir)
-    file = aprintf ("%s/.wgetrc", opt.homedir);
-#endif /* def __VMS [else] */
+  /* Join opt.homedir ($HOME) and ".wgetrc" */
+  if (opt.homedir) {
+    file = ajoin_dir_file(opt.homedir, ".wgetrc");
+  }
 
   if (!file)
     return NULL;
@@ -827,6 +844,8 @@ parse_line (const char *line, char **com, char **val, int *comind)
   const char *end = line + strlen (line);
   const char *cmdstart, *cmdend;
   const char *valstart, *valend;
+  char buf[1024];
+  size_t len;
 
   char *cmdcopy;
   int ind;
@@ -867,9 +886,18 @@ parse_line (const char *line, char **com, char **val, int *comind)
 
   /* The line now known to be syntactically correct.  Check whether
      the command is valid.  */
-  BOUNDED_TO_ALLOCA (cmdstart, cmdend, cmdcopy);
+  len = cmdend - cmdstart;
+  if (len < sizeof (buf))
+    cmdcopy = buf;
+  else
+    cmdcopy = xmalloc (len + 1);
+  memcpy (cmdcopy, cmdstart, len);
+  cmdcopy[len] = 0;
+
   dehyphen (cmdcopy);
   ind = command_by_name (cmdcopy);
+  if (cmdcopy != buf)
+    xfree (cmdcopy);
   if (ind == -1)
     return line_unknown_command;
 
@@ -946,12 +974,13 @@ void
 setoptval (const char *com, const char *val, const char *optname)
 {
   /* Prepend "--" to OPTNAME. */
-  char *dd_optname = (char *) alloca (2 + strlen (optname) + 1);
-  dd_optname[0] = '-';
-  dd_optname[1] = '-';
-  strcpy (dd_optname + 2, optname);
+  char dd_optname[2 + MAX_LONGOPTION + 1];
+
+  if ((unsigned) snprintf(dd_optname, sizeof (dd_optname), "--%s", optname) > sizeof (dd_optname))
+    exit (WGET_EXIT_PARSE_ERROR);
 
   assert (val != NULL);
+
   if (!setval_internal (command_by_name (com), dd_optname, val))
     exit (WGET_EXIT_PARSE_ERROR);
 }
@@ -1409,6 +1438,13 @@ cmd_time (const char *com, const char *val, void *place)
   if (!simple_atof (val, end, &number))
     goto err;
 
+  if (number < 0)
+    {
+      fprintf (stderr, _("%s: %s: Negative time period %s\n"),
+               exec_name, com, quote (val));
+      return false;
+    }
+
   *(double *)place = number * mult;
   return true;
 }
@@ -1929,6 +1965,10 @@ cleanup (void)
   host_cleanup ();
   log_cleanup ();
   netrc_cleanup ();
+#ifdef HAVE_SSL
+  ssl_cleanup ();
+#endif
+  connect_cleanup ();
 
   xfree (opt.choose_config);
   xfree (opt.lfilename);
@@ -2005,7 +2045,9 @@ cleanup (void)
 
   xfree (opt.encoding_remote);
   xfree (opt.locale);
+#ifdef HAVE_HSTS
   xfree (opt.hsts_file);
+#endif
 
   xfree (opt.wgetrcfile);
   xfree (opt.homedir);
