@@ -1,5 +1,5 @@
 /* Declarations for init.c.
-   Copyright (C) 1996-2011, 2015, 2018-2019 Free Software Foundation,
+   Copyright (C) 1996-2011, 2015, 2018-2020 Free Software Foundation,
    Inc.
 
 This file is part of GNU Wget.
@@ -31,6 +31,7 @@ as that of the covered work.  */
 #ifndef INIT_H
 #define INIT_H
 
+char *ajoin_dir_file (const char *, const char *);
 char *wgetrc_env_file_name (void);
 char *wgetrc_user_file_name (void);
 char *wgetrc_file_name (void);
@@ -42,4 +43,6 @@ void cleanup (void);
 void defaults (void);
 bool run_wgetrc (const char *file, file_stats_t *);
 
+#define MAX_LONGOPTION 26
+
 #endif /* INIT_H */
