@@ -1,5 +1,5 @@
 /* Messages logging.
-   Copyright (C) 1998-2011, 2015, 2018-2019 Free Software Foundation,
+   Copyright (C) 1998-2011, 2015, 2018-2020 Free Software Foundation,
    Inc.
 
 This file is part of GNU Wget.
@@ -427,6 +427,9 @@ log_vprintf_internal (struct logvprintf_state *state, const char *fmt,
   FILE *fp = get_log_fp ();
   FILE *warcfp = get_warc_log_fp ();
 
+  if (fp == NULL)
+      return false;
+
   if (!save_context_p && warcfp == NULL)
     {
       /* In the simple case just call vfprintf(), to avoid needless
@@ -480,7 +483,7 @@ log_vprintf_internal (struct logvprintf_state *state, const char *fmt,
   if (save_context_p)
     saved_append (write_ptr);
   FPUTS (write_ptr, fp);
-  if (warcfp != NULL)
+  if (warcfp != NULL && warcfp != fp)
     FPUTS (write_ptr, warcfp);
   xfree (state->bigmsg);
 
@@ -901,6 +904,7 @@ escnonprint_uri (const char *str)
   return escnonprint_internal (str, '%', 16);
 }
 
+#if defined DEBUG_MALLOC || defined TESTING
 void
 log_cleanup (void)
 {
@@ -908,6 +912,7 @@ log_cleanup (void)
   for (i = 0; i < countof (ring); i++)
     xfree (ring[i].buffer);
 }
+#endif
 
 /* When SIGHUP or SIGUSR1 are received, the output is redirected
    elsewhere.  Such redirection is only allowed once. */
@@ -966,7 +971,7 @@ redirect_output (bool to_file, const char *signal_name)
 static void
 check_redirect_output (void)
 {
-#ifndef WINDOWS
+#if !defined(WINDOWS) && !defined(__VMS)
   /* If it was redirected already to log file by SIGHUP, SIGUSR1 or -o parameter,
    * it was permanent.
    * If there was no SIGHUP or SIGUSR1 and shell is interactive
@@ -986,5 +991,5 @@ check_redirect_output (void)
           redirect_output (false,NULL);
         }
     }
-#endif /* WINDOWS */
+#endif /* !defined(WINDOWS) && !defined(__VMS) */
 }
