@@ -1,5 +1,5 @@
 /* Metalink module.
-   Copyright (C) 2015, 2018-2019 Free Software Foundation, Inc.
+   Copyright (C) 2015, 2018-2020 Free Software Foundation, Inc.
 
 This file is part of GNU Wget.
 
@@ -40,7 +40,7 @@ as that of the covered work.  */
 #include "sha1.h"
 #include "sha256.h"
 #include "sha512.h"
-#include "dosname.h"
+#include "filename.h"
 #include "xmemdup0.h"
 #include "xstrndup.h"
 #include "c-strcase.h"
@@ -1108,7 +1108,7 @@ badhash_suffix (char *name)
   char *bhash, *uname;
 
   bhash = concat_strings (name, ".badhash", (char *)0);
-  uname = unique_name (bhash, false);
+  uname = unique_name (bhash);
 
   logprintf (LOG_VERBOSE, _("Renaming %s to %s.\n"),
              quote_n (0, name), quote_n (1, uname));
