@@ -1,5 +1,5 @@
 /* Declarations for metalink.c.
-   Copyright (C) 2015 Free Software Foundation, Inc.
+   Copyright (C) 2015, 2018-2020 Free Software Foundation, Inc.
 
 This file is part of GNU Wget.
 
@@ -30,6 +30,7 @@ as that of the covered work.  */
 #define	METALINK_H
 
 #include <metalink/metalink_types.h>
+#include "dirname.h"
 #include "wget.h"
 
 #ifdef HAVE_SSL
@@ -50,7 +51,6 @@ int metalink_meta_cmp (const void* meta1, const void* meta2);
 
 int metalink_check_safe_path (const char *path);
 
-char *last_component (char const *name);
 void replace_metalink_basename (char **name, char *ref);
 char *get_metalink_basename (char *name);
 void append_suffix_number (char **str, const char *sep, wgint num);
