@@ -1,5 +1,5 @@
 /* Declarations for windows
-   Copyright (C) 1996-2011, 2015, 2018-2019 Free Software Foundation,
+   Copyright (C) 1996-2011, 2015, 2018-2020 Free Software Foundation,
    Inc.
 
 This file is part of GNU Wget.
@@ -59,32 +59,8 @@ as that of the covered work.  */
 /* Declares inet_ntop() and inet_pton(). */
 #include <arpa/inet.h>
 
-/* We have strcasecmp and strncasecmp, just under different names.  */
-#ifndef HAVE_STRCASECMP
-# define strcasecmp stricmp
-#endif
-#ifndef HAVE_STRNCASECMP
-# define strncasecmp strnicmp
-#endif
-
 #include <stdio.h>
 
-/* Define a wgint type under Windows. */
-typedef __int64 wgint;
-#define SIZEOF_WGINT 8
-
-/* str_to_wgint is a function with the semantics of strtol[l], but
-   which works on wgint.  */
-#if defined HAVE_STRTOLL
-# define str_to_wgint strtoll
-#elif defined HAVE__STRTOI64
-# define str_to_wgint _strtoi64
-#else
-# define str_to_wgint strtoll
-# define NEED_STRTOLL
-# define strtoll_type __int64
-#endif
-
 #define PATH_SEPARATOR '\\'
 
 /* ioctl needed by set_windows_fd_as_blocking_socket() */
